package qsrtdemo.control;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Observable;
import java.util.Observer;
import java.util.regex.Pattern;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.Border;
import javax.swing.border.LineBorder;
import qsrtdemo.model.InitSort;
import qsrtdemo.model.Quicksort;
import qsrtdemo.model.RandomiseVars;

/**
 * This is the panel at the very bottom of the screen where there are buttons
 * and inputs to set up the array and step through the sorting process.
 *
 * @author Michael Hawes
 */
public class ControlPanel extends JPanel implements ActionListener {
    
    public static final Integer PREV_STEP = new Integer(0);
    public static final Integer NEXT_STEP = new Integer(1);
    public static final Integer FIRST_STEP = new Integer(2);
    public static final Integer FINAL_STEP = new Integer(3);
    public static final Integer PREV_PART = new Integer(4);
    public static final Integer NEXT_PART = new Integer(5);
    public static final Integer PLAY_STEPS = new Integer(6);
    public static final Integer STOP_PLAY = new Integer(7);
    public static final int DEF_MAXVAL = 100;
    public static final int DEF_MINVAL = 0;
    public static final int DEF_NUMVALS = 10;
    public static final int STEP_TIME = 2500;
    private static final Pattern arrayPatt = Pattern.compile("(-?[0-9]+ )*-?[0-9]+");
    private static final Border validBorder = new LineBorder(Color.BLACK);
    private static final Border invalidBorder = new LineBorder(Color.RED);
    
    private ControlPanelMessenger controlPanelMessenger;
    
    private JButton first;
    private JButton prevPart;
    private JButton previous;
    private JButton play;
    private JButton stop;
    private JButton next;
    private JButton nextPart;
    private JButton last;
    private JButton setArray;
    private JButton randArray;
    private JTextField arrayField;
    private JTextField maxArrayVal;
    private JTextField minArrayVal;
    private JTextField numArrayVals;
    private InitSort init;
    
    public ControlPanel() {
        
        controlPanelMessenger = new ControlPanelMessenger();
        
        this.setLayout(new BorderLayout());
        
        first = new JButton("|<");
        first.setToolTipText("To first step in sort");
        prevPart = new JButton("<<");
        prevPart.setToolTipText("To previous partition");
        previous = new JButton("<");
        previous.setToolTipText("To previous step");
        play = new JButton("Play");
        play.setToolTipText("Play through the sorting process");
        stop = new JButton("Stop");
        stop.setToolTipText("Stop playing");
        next = new JButton(">");
        next.setToolTipText("To next step");
        nextPart = new JButton(">>");
        nextPart.setToolTipText("To next partition");
        last = new JButton(">|");
        last.setToolTipText("To end of sorting process");
        
        JPanel stepPanel = new JPanel();
        stepPanel.setLayout(new FlowLayout());
        stepPanel.add(first);
        stepPanel.add(prevPart);
        stepPanel.add(previous);
        stepPanel.add(play);
        stepPanel.add(stop);
        stepPanel.add(next);
        stepPanel.add(nextPart);
        stepPanel.add(last);
        
        setArray = new JButton("Set Array");
        setArray.setToolTipText("Click here to set the array as shown in the box to the left");
        arrayField = new JTextField();
        arrayField.setBorder(validBorder);
        arrayField.setToolTipText("Type numbers here to appear in the array, then click the button to the right");
        
        randArray = new JButton("Randomise Array");
        randArray.setToolTipText("Click here to randomise the array within the given parameters");
        maxArrayVal = new JTextField(""+DEF_MAXVAL);
        maxArrayVal.setBorder(validBorder);
        maxArrayVal.setToolTipText("Enter the maximum value to randomly generate here");
        JLabel maxArrayValLabel = new JLabel("Max Value:");
        minArrayVal = new JTextField(""+DEF_MINVAL);
        minArrayVal.setBorder(validBorder);
        minArrayVal.setToolTipText("Enter the minimum value to randomly generate here");
        JLabel minArrayValLabel = new JLabel("Min Value:");
        numArrayVals = new JTextField(""+DEF_NUMVALS);
        numArrayVals.setBorder(validBorder);
        numArrayVals.setToolTipText("Enter the amount of numbers to randomly generate here");
        JLabel numArrayValsLabel = new JLabel("Length of array:");
        
        JPanel arrayPanel = new JPanel();
        arrayPanel.setLayout(new GridBagLayout());
        GridBagConstraints co = new GridBagConstraints();
        co.anchor=GridBagConstraints.LINE_END;
        co.fill=GridBagConstraints.HORIZONTAL;
        co.weightx=0.99;
        co.gridx=0;
        co.gridy=0;
        co.gridwidth=6;
        co.insets = new Insets(0,5,5,10);
        arrayPanel.add(arrayField, co);
        co.insets.set(0,0,5,10);
        co.gridy=1;
        co.gridwidth=1;
        co.gridx=1;
        arrayPanel.add(numArrayVals, co);
        co.gridx=3;
        arrayPanel.add(maxArrayVal, co);
        co.gridx=5;
        arrayPanel.add(minArrayVal, co);
        co.fill=GridBagConstraints.NONE;
        co.insets.set(0,5,5,5);
        co.weightx=0.01;
        co.gridx=0;
        arrayPanel.add(numArrayValsLabel, co);
        co.insets.set(0,0,5,5);
        co.gridx=2;
        arrayPanel.add(maxArrayValLabel, co);
        co.gridx=4;
        arrayPanel.add(minArrayValLabel, co);
        co.insets.set(0,0,5,5);
        co.anchor=GridBagConstraints.LINE_START;
        co.gridx=6;
        arrayPanel.add(randArray, co);
        co.gridy=0;
        co.insets.set(0,0,5,5);
        arrayPanel.add(setArray, co);
        
        first.addActionListener(this);
        prevPart.addActionListener(this);
        previous.addActionListener(this);
        play.addActionListener(this);
        stop.addActionListener(this);
        next.addActionListener(this);
        nextPart.addActionListener(this);
        last.addActionListener(this);
        setArray.addActionListener(this);
        randArray.addActionListener(this);
        
        first.setEnabled(false);
        prevPart.setEnabled(false);
        previous.setEnabled(false);
        play.setEnabled(false);
        stop.setEnabled(false);
        next.setEnabled(false);
        nextPart.setEnabled(false);
        last.setEnabled(false);
        
        this.add(stepPanel, BorderLayout.NORTH);
        this.add(arrayPanel, BorderLayout.CENTER);
        
    }
    
    public void addObserver(Observer o) {
        
        controlPanelMessenger = new ControlPanelMessenger();
        controlPanelMessenger.addObserver(o);
        
    }
    
    private boolean makeInit() {
        String arrayString = arrayField.getText();
        if(arrayPatt.matcher(arrayString).matches()) {
            String[] numStrings = arrayField.getText().split(" ");
            int[] array = new int[numStrings.length];
            for(int i=0; i<numStrings.length; i++) {
                array[i] = Integer.parseInt(numStrings[i]);
            }
            init = new InitSort(array, Quicksort.ORDER_DESCENDING);
            arrayField.setBorder(validBorder);
            return true;
        } else {
            arrayField.setBorder(invalidBorder);
            return false;
        }
    }
    
    private boolean makeInitRand() {
        RandomiseVars random=null;
        try {
            int maxVal = Integer.parseInt(maxArrayVal.getText());
            maxArrayVal.setBorder(validBorder);
            
            try {
                int numVals = Integer.parseInt(numArrayVals.getText());
                numArrayVals.setBorder(validBorder);
                
                int minVal;
                try {
                    minVal = Integer.parseInt(minArrayVal.getText());
                } catch(NumberFormatException e) {
                    minArrayVal.setText(""+DEF_MINVAL);
                    minVal = DEF_MINVAL;
                }
                
                random = new RandomiseVars(numVals, maxVal, minVal);
                init = new InitSort(random, Quicksort.ORDER_DESCENDING);
                int[] array = init.getArray();
                String arrayTxt="";
                if(array.length>0) {
                    arrayTxt += array[0];
                    for(int i=1; i<array.length; i++) {
                        arrayTxt += " "+array[i];
                    }
                }
                arrayField.setText(arrayTxt);
                arrayField.setBorder(validBorder);
                return true;
            } catch(NumberFormatException e) {
                numArrayVals.setBorder(invalidBorder);
                return false;
            }
        } catch(NumberFormatException e) {
            maxArrayVal.setBorder(invalidBorder);
            try {
                Integer.parseInt(numArrayVals.getText());
                numArrayVals.setBorder(validBorder);
            } catch(NumberFormatException e2) {
                numArrayVals.setBorder(invalidBorder);
            }
            return false;
        }
    }
    
    private void initSort() {
        if ((controlPanelMessenger==null) || (controlPanelMessenger.countObservers()==0)) {
            System.out.println("Add an observer to ControlPanel first..");
        } else {
            controlPanelMessenger.makeChanged();
            controlPanelMessenger.notifyObservers(init);
        }
    }
    
    public void actionPerformed(ActionEvent e) {
        
        Object source = e.getSource();
        
        if (source == play) {
            
            controlPanelMessenger.makeChanged();
            controlPanelMessenger.notifyObservers(PLAY_STEPS);
            
        } else if (source == stop) {

            controlPanelMessenger.makeChanged();
            controlPanelMessenger.notifyObservers(STOP_PLAY);
            
        } else if (source == next) {
            
            controlPanelMessenger.makeChanged();
            controlPanelMessenger.notifyObservers(NEXT_STEP);
            
        } else if (source == previous) {
            
            controlPanelMessenger.makeChanged();
            controlPanelMessenger.notifyObservers(PREV_STEP);
            
        } else if (source == first) {
            
            controlPanelMessenger.makeChanged();
            controlPanelMessenger.notifyObservers(FIRST_STEP);
            
        } else if (source == last) {
            
            controlPanelMessenger.makeChanged();
            controlPanelMessenger.notifyObservers(FINAL_STEP);
            
        } else if (source == prevPart) {
            
            controlPanelMessenger.makeChanged();
            controlPanelMessenger.notifyObservers(PREV_PART);
            
        } else if (source == nextPart) {
            
            controlPanelMessenger.makeChanged();
            controlPanelMessenger.notifyObservers(NEXT_PART);
            
        } else if (source == setArray) {
            
            if(makeInit()) {
                initSort();
                enableStepButtons();
            }
            
        } else if (source == randArray) {
            
            if(makeInitRand()) {
                initSort();
                enableStepButtons();
            }
            
        }
        
    }
    
    private void enableStepButtons() {
        first.setEnabled(true);
        prevPart.setEnabled(true);
        previous.setEnabled(true);
        play.setEnabled(true);
        stop.setEnabled(true);
        next.setEnabled(true);
        nextPart.setEnabled(true);
        last.setEnabled(true);
    }
    
    //------------------------------------------------------------------------->
    
    private class ControlPanelMessenger extends Observable {
        
        private ControlPanelMessenger() {
        }
        
        public void makeChanged() {
            super.setChanged();
        }
        
    }
    
}