package qsrtdemo.view;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Observable;
import java.util.Observer;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.EtchedBorder;
import qsrtdemo.model.PartitionStep;
import qsrtdemo.model.Quicksort;
import qsrtdemo.model.StartPartStep;
import qsrtdemo.model.SwapStep;
import qsrtdemo.model.ValStep;

/**
 *
 * @author Michael Hawes
 */
public class DescriptionPanel extends JPanel implements Observer{
    private static final String welcomeMsg = "Welcome to the Quicksort demonstration.";
    private static final Color enabledColour = new Color(224, 224, 224);
    private static final Color disabledColour = new Color(200, 200, 200);
    
    private JLabel stepLabel;
    private JLabel partLabel;
    private JLabel descLabel;
    private JPanel stepPanel;
    private JPanel descPanel;
    private String stepDesc;
    private String partDesc;
    private String shortStepDesc;
    private String shortPartDesc;
    private String desc;
    private JButton button;
    private boolean shown=true;
    private boolean started=false;
    private boolean sortDescending;
    
    //private int counter = 0;
    public DescriptionPanel() {
        
        // label settings
        stepLabel = new JLabel(" ", JLabel.CENTER);
        partLabel = new JLabel(welcomeMsg, JLabel.CENTER);
        descLabel = new JLabel(" ", JLabel.CENTER);
        stepPanel = new JPanel(new BorderLayout());
        stepPanel.add(stepLabel, BorderLayout.NORTH);
        stepPanel.add(partLabel, BorderLayout.CENTER);
        descPanel = new JPanel(new BorderLayout());
        descPanel.add(stepPanel, BorderLayout.NORTH);
        descPanel.add(descLabel, BorderLayout.CENTER);
        
        // panel settings
        setBorder(BorderFactory.createEtchedBorder(EtchedBorder.LOWERED, Color.white, Color.gray));
        setLayout(new BorderLayout());
        setLabelsColour(enabledColour);
        add(descPanel, BorderLayout.CENTER);
        
        // button initialization
        button = new JButton("Hide description");
        button.setToolTipText("Hide description to the left");
        button.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e) {
                if(shown){
                    // hide description
                    shown = false;
                    setLabelsColour(disabledColour);
                    if(started) {
                        stepLabel.setText(shortStepDesc);
                        partLabel.setText(shortPartDesc);
                    } else {
                        stepLabel.setText(" ");
                        partLabel.setText(" ");
                    }
                    descLabel.setText(" ");
                    button.setText("Show description");
                    button.setToolTipText("Show description to the left");
                } else {
                    // show description
                    shown = true;
                    setLabelsColour(enabledColour);
                    if(started) {
                        stepLabel.setText(stepDesc);
                        partLabel.setText(partDesc);
                        descLabel.setText(desc);
                    } else {
                        stepLabel.setText(" ");
                        partLabel.setText(welcomeMsg);
                        descLabel.setText(" ");
                    }
                    button.setText("Hide description");
                    button.setToolTipText("Hide description to the left");
                }
            }
        });
        add(button, BorderLayout.EAST);
    }
    
    private void setLabelsColour(Color colour) {
        setBackground(colour);
        stepLabel.setBackground(colour);
        partLabel.setBackground(colour);
        descLabel.setBackground(colour);
        stepPanel.setBackground(colour);
        descPanel.setBackground(colour);
    }
    
    public void update(Observable o, Object arg) {
        Quicksort sort = (Quicksort)o;
        if(arg == Quicksort.UPDATE_BEGIN) {
            if(sort.getSortOrder()==Quicksort.ORDER_DESCENDING) {
                sortDescending=true;
            } else sortDescending=false;
        }

        PartitionStep step = sort.getCurrStep();
        if(step.getClass().equals(StartPartStep.class)){
            desc = "Starting new partition, choosing pivot";
        }else if(step.getClass().equals(SwapStep.class)){
            SwapStep ss = (SwapStep)step;
            int swapType = ss.getSwapType();
            if(swapType == SwapStep.SWAP_MAKEPIVOTSAFE) {
                desc = "Moving pivot to safety at end of partition";
            } else if(swapType == SwapStep.SWAP_SWAPVALUES) {
                if(sortDescending) {
                    desc = "Found value lower than pivot before value higher than pivot, swapping them";
                } else {
                    desc = "Found value higher than pivot before value lower than pivot, swapping them";
                }
            } else if(swapType == SwapStep.SWAP_MAKEPIVOTCORRECT) {
                desc = "Moving pivot to its final position in the array between " +
                          "all values higher than it and all values lower than it";
            } else if(swapType == SwapStep.SWAP_PIVOTSTART) {
                if(sortDescending) {
                    desc = "Found no values higher than the pivot in the partition, " +
                              "moving it to the beginning of the partition";
                } else {
                    desc = "Found no values lower than the pivot in the partition, " +
                              "moving it to the beginning of the partition";
                }
            } else {
                desc = "Invalid swap step type";
            }
        }else if(step.getClass().equals(ValStep.class)){
            ValStep vs = (ValStep)step;
            int type = vs.getValType();
            if(type == ValStep.VAL_PIVOTEND) {
                if(sortDescending) {
                    desc = "Found no values lower than the pivot in the partition, " +
                              "leaving it at the end of the partition";
                } else {
                    desc = "Found no values higher than the pivot in the partition, " +
                              "leaving it at the end of the partition";
                }
            } else if(type == ValStep.VAL_SINGLEVAL) {
                desc = "Partition consists of only a single value, leaving it as it is";
            } else {
                desc = "Invalid single value step type";
            }
        } else {
            desc = "Invalid step type";
        }

        stepDesc = "Step "+(sort.getCurrStepIndexInPart()+1)+" of "+sort.getCurrPartNumSteps()+" in partition\n";
        partDesc = "Partition "+(sort.getCurrPartIndex()+1)+" of "+sort.getNumParts()+" in sort";
        shortStepDesc = "Step "+(sort.getCurrStepIndexInPart()+1)+" in partition\n";
        shortPartDesc = "Partition "+(sort.getCurrPartIndex()+1)+" in sort";

        if(!started) started = true;

        if(shown) {
            stepLabel.setText(stepDesc);
            partLabel.setText(partDesc);
            descLabel.setText(desc);
        } else {
            stepLabel.setText(shortStepDesc);
            partLabel.setText(shortPartDesc);
            descLabel.setText(" ");
        }
    }
    
}