package qsrtdemo.view;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.font.FontRenderContext;
import java.awt.geom.Rectangle2D;
import qsrtdemo.model.Partition;

/**
 *
 * @author Michael Hawes
 */
public class PartitionNode {
    
    private static final int yPad=5;
    private static final int xPad=5;
    private static int xStart=0;
    private static int yStart=25;
    private static int partSelected=-1;
    private static PartitionNode base;
    private static Font font;
    private static FontRenderContext context;
    
    private static final int yGap=10;
    private static final int xGap=10;
    private static final int width=20;
    private static final int height=20;
    private static final int pivotYGap=25;

    private static final Color leftColour = new Color(148, 0, 211);
    private static final Color rightColour = new Color(30, 144, 255);
    private static final Color unexecutedColour = Color.lightGray;
    
    private Partition part;
    private int partIndex;
    private String str;
    private float strX;
    private float strY;
    private String pivotStr;
    private float pivotStrX;
    private float pivotStrY;
    private int depth;
    private PartitionNode leftNode;
    private PartitionNode rightNode;
    private int maxPartIndex;
    private int maxDepth;
    private int x=-1;
    private int y;
    private int spaceLeft;
    private int spaceRight;
    private boolean spaceCalced=false;
    
    public static void setPartSelected(int index) {
        partSelected = index;
    }
    public static void init(Partition part, Font font, FontRenderContext context, int yOffset) {
        PartitionNode.font = font;
        PartitionNode.context = context;
        PartitionNode.yStart = yOffset;
        
        base = new PartitionNode(part, font, context);
        partSelected = 0;
    }
    public static PartitionNode getBase() {
        return base;
    }
    
    public PartitionNode(Partition part, Font font, FontRenderContext context) {
        initNode(part, 0, 0, font, context);
        
        calcSpace();
        initX(xStart+xPad);
        initStr(font, context);
    }
    
    private PartitionNode(Partition part, int partIndex, int depth, Font font, FontRenderContext context) {
        initNode(part, partIndex, depth, font, context);
    }
    
    private void initNode(Partition part, int partIndex, int depth, Font font, FontRenderContext context) {
        this.part = part;
        this.partIndex = partIndex;
        str = ""+(partIndex+1);
        pivotStr = ""+this.part.getPivot();
        maxPartIndex = partIndex;
        this.depth = depth;
        maxDepth = depth;
        y = yPad+yStart+depth*(yGap+height);
        
        int newDepth = depth+1;
        
        Partition left = part.getLeftPartition();
        if(left!=null) {
            leftNode = new PartitionNode(left, maxPartIndex+1, newDepth, font, context);
            maxPartIndex = leftNode.getMaxPartNum();
            maxDepth = leftNode.getMaxDepth();
        }
        
        Partition right = part.getRightPartition();
        if(right!=null) {
            rightNode = new PartitionNode(right, maxPartIndex+1, newDepth, font, context);
            maxPartIndex = rightNode.getMaxPartNum();
            int rightDepth = rightNode.getMaxDepth();
            if(rightDepth>maxDepth) maxDepth = rightDepth;
        }
    }
    
    private boolean initStr(Font font, FontRenderContext context) {
        if(x>0) {
            Rectangle2D bounds = font.createGlyphVector(context, str).getVisualBounds();
            float strHeight = (float)bounds.getHeight();
            float strWidth = (float)bounds.getWidth();
            strY = y+(float)((strHeight+(float)height)/2.0);
            strX = x+(float)(((float)width-strWidth)/2.0);

            Rectangle2D pivotBounds = font.createGlyphVector(context, pivotStr).getVisualBounds();
            float pivotStrHeight = (float)pivotBounds.getHeight();
            float pivotStrWidth = (float)pivotBounds.getWidth();
            pivotStrY = y-pivotYGap+(float)((pivotStrHeight+(float)height)/2.0);
            pivotStrX = x+(float)(((float)width-pivotStrWidth)/2.0);
            
            if(leftNode!=null) leftNode.initStr(font, context);
            if(rightNode!=null) rightNode.initStr(font, context);
            return true;
        } else return false;
    }
    
    private void initX(int start) {
        if(spaceCalced) {
            x = start+spaceLeft;
            if(leftNode!=null) {
                if(rightNode!=null) {
                    leftNode.initX(start);
                    int left = xGap;
                    int right = width+xGap;
                    int gap = xGap*2+width;
                    if(leftNode.getSpaceRight()>right || rightNode.getSpaceLeft()>left) {
                        int space = rightNode.getSpaceLeft()+leftNode.getSpaceRight()+xGap;
                        if(space>gap) {
                            rightNode.initX(start+leftNode.getSpaceLeft()+leftNode.getSpaceRight()+xGap);
                        } else {
                            rightNode.initX(x+gap-rightNode.getSpaceLeft());
                        }
                    } else {
                        rightNode.initX(x+right-rightNode.getSpaceLeft());
                    }
                } else {
                    leftNode.initX(start);
                }
            } else {
                if(rightNode!=null) {
                    rightNode.initX(x+width+xGap-rightNode.getSpaceLeft());
                }
            }
        }
    }
    
    private void calcSpace() {
        if(leftNode!=null) {
            if(rightNode!=null) {
                leftNode.calcSpace();
                rightNode.calcSpace();
                spaceLeft = xGap;
                spaceRight = width+xGap;
                int gap = xGap*2+width;
                if(rightNode.getSpaceLeft()>spaceRight || leftNode.getSpaceRight()>spaceLeft) {
                    int space = rightNode.getSpaceLeft()+leftNode.getSpaceRight()+xGap;
                    if(space>gap) {
                        space = (int)(space/2.0);
                        spaceLeft = space+leftNode.getSpaceLeft();
                        spaceRight = space+rightNode.getSpaceRight();
                    } else {
                        spaceLeft = spaceLeft+leftNode.getSpaceLeft()+width;
                        spaceRight = spaceRight+rightNode.getSpaceRight();
                    }
                } else {
                    spaceLeft = spaceLeft+leftNode.getSpaceLeft()+width;
                    spaceRight = spaceRight+rightNode.getSpaceRight();
                }
            } else {
                leftNode.calcSpace();
                spaceLeft = leftNode.getSpaceLeft()+width+xGap;
                spaceRight = leftNode.getSpaceRight()-width-xGap;
                if(spaceRight<width) spaceRight = width;
            }
        } else {
            if(rightNode!=null) {
                rightNode.calcSpace();
                spaceRight = rightNode.getSpaceRight()+width+xGap;
                spaceLeft = rightNode.getSpaceLeft()-width-xGap;
                if(spaceLeft<0) {
                    spaceLeft = 0;
                }
            } else {
                spaceLeft=0;
                spaceRight=width;
            }
        }
        spaceCalced=true;
    }
    
    public int getSpaceLeft() {
        return spaceLeft;
    }
    
    public int getSpaceRight() {
        return spaceRight;
    }
    
    public int getMaxPartNum() {
        return maxPartIndex;
    }
    
    public int getMaxDepth() {
        return maxDepth;
    }
    
    public int getCentreX() {
        return x+(width/2);
    }
    public int getCentreY() {
        return y+(height/2);
    }
    
    public Dimension getBounds() {
        return new Dimension(xPad*2+spaceLeft+spaceRight, yPad*2+(maxDepth+1)*height+maxDepth*yGap);
    }
    
    public void setXStart(int newX) {
        int old = xStart;
        xStart = newX;
        int diff = xStart - old;
        moveX(diff);
    }
    public void moveX(int move) {
        x+=move;
        strX+=move;
        pivotStrX+=move;
        if(leftNode!=null) leftNode.moveX(move);
        if(rightNode!=null) rightNode.moveX(move);
    }

    public void drawPivot(Graphics2D g) {
        g.setColor(Color.white);
        g.fillRect(x, y-pivotYGap, width, height);
        g.setColor(Color.blue);
        g.drawRect(x, y-pivotYGap, width, height);
        g.setColor(Color.black);
        g.drawString(pivotStr, pivotStrX, pivotStrY);
    }
    
    public void draw(Graphics2D g) {
        if(partSelected==partIndex) {
            drawPivot(g);
            g.setColor(Color.orange);
        } else {
            if(partIndex<partSelected) {
                if(leftNode!=null) {
                    g.setColor(leftColour);
                    g.drawLine(getCentreX(), getCentreY(), leftNode.getCentreX(), leftNode.getCentreY());
                    leftNode.draw(g);
                }
                if(rightNode!=null) {
                    g.setColor(rightColour);
                    g.drawLine(getCentreX(), getCentreY(), rightNode.getCentreX(), rightNode.getCentreY());
                    rightNode.draw(g);
                }
                drawPivot(g);
            }
            g.setColor(Color.white);
        }
        g.fillOval(x, y, width, height);
        
        if(partIndex>partSelected) {
            g.setColor(unexecutedColour);
            g.drawOval(x, y, width, height);
            g.setColor(Color.black);
        } else {
            g.setColor(Color.black);
            g.drawOval(x, y, width, height);
            g.drawString(str, strX, strY);
        }
    }

    public void drawLeft(Graphics2D g) {
        if(leftNode!=null) {
            g.setColor(leftColour);
            g.drawLine(getCentreX(), getCentreY(), leftNode.getCentreX(), leftNode.getCentreY());
            leftNode.drawLeft(g);
        }
        if(rightNode!=null) {
            g.setColor(leftColour);
            g.drawLine(getCentreX(), getCentreY(), rightNode.getCentreX(), rightNode.getCentreY());
            rightNode.drawLeft(g);
        }

        if(partSelected==partIndex) {
            g.setColor(Color.orange);
        } else {
            g.setColor(Color.white);
        }
        g.fillOval(x, y, width, height);
        g.setColor(leftColour);
        g.drawOval(x, y, width, height);

        g.setColor(Color.black);
        g.drawString(str, strX, strY);
    }

    public void drawRight(Graphics2D g) {
        if(leftNode!=null) {
            g.setColor(rightColour);
            g.drawLine(getCentreX(), getCentreY(), leftNode.getCentreX(), leftNode.getCentreY());
            leftNode.drawRight(g);
        }
        if(rightNode!=null) {
            g.setColor(rightColour);
            g.drawLine(getCentreX(), getCentreY(), rightNode.getCentreX(), rightNode.getCentreY());
            rightNode.drawRight(g);
        }

        if(partSelected==partIndex) {
            g.setColor(Color.orange);
        } else {
            g.setColor(Color.white);
        }
        g.fillOval(x, y, width, height);
        g.setColor(rightColour);
        g.drawOval(x, y, width, height);

        g.setColor(Color.black);
        g.drawString(str, strX, strY);
    }
    
    public int getPartNumAt(int xCheck, int yCheck) {
        if(xCheck>=x && xCheck<=(x+width) && yCheck>=y && yCheck<=(y+height)) {
            return partIndex;
        } else {
            int index = -1;
            if(leftNode!=null) {
                index = leftNode.getPartNumAt(xCheck, yCheck);
            }
            if(index==-1 && rightNode!=null) {
                index = rightNode.getPartNumAt(xCheck, yCheck);
            }
            return index;
        }
    }
    
}
